from datetime import timedelta

import pytest

from .conf import TlsTestConf
from pyhttpd.env import HttpdTestEnv

@pytest.mark.skipif(condition=not HttpdTestEnv.has_shared_module("tls"), reason="no mod_tls available")

class TestProxyTLS:

    @pytest.fixture(autouse=True, scope='class')
    def _class_scope(self, env):
        # add vhosts a+b and a ssl proxy from a to b
        conf = TlsTestConf(env=env, extras={
            'base': [
                "LogLevel proxy:trace1 proxy_http:trace1 proxy_http2:trace2 http2:trace2 cgid:trace4",
                "TLSProxyProtocol TLSv1.3+",
                f"<Proxy https://127.0.0.1:{env.https_port}/>",
                "    TLSProxyEngine on",
                f"    TLSProxyCA {env.ca.cert_file}",
                "    TLSProxyProtocol TLSv1.2+",
                "    TLSProxyCiphersPrefer TLS13_AES_256_GCM_SHA384",
                "    TLSProxyCiphersSuppress TLS13_AES_128_GCM_SHA256",
                "    ProxyPreserveHost on",
                "</Proxy>",
                f"<Proxy https://localhost:{env.https_port}/>",
                "    ProxyPreserveHost on",
                "</Proxy>",
                f"<Proxy h2://127.0.0.1:{env.https_port}/>",
                "    TLSProxyEngine on",
                f"    TLSProxyCA {env.ca.cert_file}",
                "    TLSProxyCiphersSuppress TLS_AES_256_GCM_SHA384:TLS_AES_128_GCM_SHA256",
                "    ProxyPreserveHost on",
                "</Proxy>",
            ],
            env.domain_b: [
                "Protocols h2 http/1.1",
                f"ProxyPass /proxy-tls/ https://127.0.0.1:{env.https_port}/",
                f"ProxyPass /proxy-local/ https://localhost:{env.https_port}/",
                f"ProxyPass /proxy-h2-tls/ h2://127.0.0.1:{env.https_port}/",
                "TLSOptions +StdEnvVars",
            ],
        })
        conf.add_tls_vhosts(domains=[env.domain_a, env.domain_b])
        conf.install()
        assert env.apache_restart() == 0

    def test_tls_15_proxy_tls_get(self, env):
        data = env.tls_get_json(env.domain_b, "/proxy-tls/index.json")
        assert data == {'domain': env.domain_b}

    def test_tls_15_proxy_tls_get_local(self, env):
        # does not work, since SSLProxy* not configured
        data = env.tls_get_json(env.domain_b, "/proxy-local/index.json")
        assert data is None
        #
        env.httpd_error_log.ignore_recent(
            lognos = [
                "AH01961",  # failed to enable ssl support [Hint: if using mod_ssl, see SSLProxyEngine]
                "AH00961"   # failed to enable ssl support (mod_proxy)
            ]
        )

    def test_tls_15_proxy_tls_h2_get(self, env):
        r = env.tls_get(env.domain_b, "/proxy-h2-tls/index.json")
        assert r.exit_code == 0
        assert r.json == {'domain': env.domain_b}, f"{r.stdout}"

    @pytest.mark.parametrize("name, value", [
        ("SERVER_NAME", "b.mod-tls.test"),
        ("SSL_PROTOCOL", "TLSv1.3"),
        ("SSL_CIPHER", "TLS_AES_256_GCM_SHA384"),
        ("SSL_SESSION_RESUMED", "Initial"),
        ("SSL_SECURE_RENEG", "false"),
        ("SSL_COMPRESS_METHOD", "NULL"),
        ("SSL_CIPHER_EXPORT", "false"),
        ("SSL_CLIENT_VERIFY", "NONE"),
    ])
    def test_tls_15_proxy_tls_h1_vars(self, env, name: str, value: str):
        r = env.tls_get(env.domain_b, f"/proxy-tls/vars.py?name={name}")
        assert r.exit_code == 0, r.stderr
        assert r.json == {name: value}, r.stdout

    @pytest.mark.parametrize("name, value", [
        ("SERVER_NAME", "b.mod-tls.test"),
        ("SSL_PROTOCOL", "TLSv1.3"),
        ("SSL_CIPHER", "TLS_CHACHA20_POLY1305_SHA256"),
        ("SSL_SESSION_RESUMED", "Initial"),
    ])
    def test_tls_15_proxy_tls_h2_vars(self, env, name: str, value: str):
        r = env.tls_get(env.domain_b, f"/proxy-h2-tls/vars.py?name={name}")
        assert r.exit_code == 0, r.stderr
        assert r.json == {name: value}, r.stdout



#!/usr/bin/env python3
import json
import os, sys
from urllib import parse
import multipart # https://github.com/andrew-d/python-multipart (`apt install python3-multipart`)


def get_request_params():
    oforms = {}
    ofiles = {}
    if "REQUEST_URI" in os.environ:
        qforms = parse.parse_qs(parse.urlsplit(os.environ["REQUEST_URI"]).query)
        for name, values in qforms.items():
            oforms[name] = values[0]
    if "HTTP_CONTENT_TYPE" in os.environ:
        ctype = os.environ["HTTP_CONTENT_TYPE"]
        if ctype == "application/x-www-form-urlencoded":
            qforms = parse.parse_qs(parse.urlsplit(sys.stdin.read()).query)
            for name, values in qforms.items():
                oforms[name] = values[0]
        elif ctype.startswith("multipart/"):
            def on_field(field):
                oforms[field.field_name] = field.value
            def on_file(file):
                ofiles[field.field_name] = field.value
            multipart.parse_form(headers={"Content-Type": ctype}, input_stream=sys.stdin.buffer, on_field=on_field, on_file=on_file)
    return oforms, ofiles


forms, files = get_request_params()

jenc = json.JSONEncoder()

def get_var(name: str, def_val: str = ""):
    if name in os.environ:
        return os.environ[name]
    return def_val

def get_json_var(name: str, def_val: str = ""):
    var = get_var(name, def_val=def_val)
    return jenc.encode(var)


name = forms['name'] if 'name' in forms else None

print("Content-Type: application/json\n")
if name:
    print(f"""{{ "{name}" : {get_json_var(name, '')}}}""")
else:
    print(f"""{{ "https" : {get_json_var('HTTPS', '')},
  "host" : {get_json_var('SERVER_NAME', '')},
  "protocol" : {get_json_var('SERVER_PROTOCOL', '')},
  "ssl_protocol" : {get_json_var('SSL_PROTOCOL', '')},
  "ssl_cipher" : {get_json_var('SSL_CIPHER', '')}
}}""")

